@@ -18,17 +18,29 @@
 /*
 boolean leap(N_int year);
 
-N_int   encode(N_int yy, N_int mm, N_int dd);
-void    decode(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd);
-boolean valid_date(N_int date);
-byteptr date_string(N_int date);
+N_int   compress(N_int yy, N_int mm, N_int dd);
+void    uncompress(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd);
+boolean check_compressed(N_int date);
+byteptr compressed_to_short(N_int date);
 
 boolean check_date(N_int year, N_int mm, N_int dd);
-long    calc_days(N_int year, N_int mm, N_int dd);
-long    dates_difference(N_int year1, N_int mm1, N_int dd1,
-                         N_int year2, N_int mm2, N_int dd2);
+Z_long  calc_days(N_int year, N_int mm, N_int dd);
 N_int   day_of_week(N_int year, N_int mm, N_int dd);
-void    calc_new_date(N_int *year, N_int *mm, N_int *dd, long offset);
+Z_long  dates_difference(N_int year1, N_int mm1, N_int dd1,
+                         N_int year2, N_int mm2, N_int dd2);
+void    calc_new_date(N_int *year, N_int *mm, N_int *dd, Z_long offset);
+
+void    date_time_difference
+(
+    Z_long *days, Z_int *hh, Z_int *mm, Z_int *ss,
+    N_int year1, N_int month1, N_int day1, N_int hh1, N_int mm1, N_int ss1,
+    N_int year2, N_int month2, N_int day2, N_int hh2, N_int mm2, N_int ss2
+);
+void    calc_new_date_time
+(
+    N_int *year, N_int *month, N_int *day, N_int *hh, N_int *mm, N_int *ss,
+    Z_long days_offset, Z_long hh_offset, Z_long mm_offset, Z_long ss_offset
+);
 
 byteptr date_to_short(N_int year, N_int mm, N_int dd);
 byteptr date_to_string(N_int year, N_int mm, N_int dd);
@@ -37,6 +49,8 @@ N_int   week_number(N_int year, N_int mm, N_int dd);
 void    first_in_week(N_int week, N_int *year, N_int *mm, N_int *dd);
 N_int   weeks_in_year(N_int year);
 
+N_int   decode_day(byteptr buffer, N_int len);
+N_int   decode_month(byteptr buffer, N_int len);
 boolean decode_date(byteptr buffer, N_int *year, N_int *mm, N_int *dd);
 */
 /**************************************/
@@ -49,34 +63,21 @@ N_int   month_length[2][13] =
         { 0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
     };
 
-N_char  day_short[7][4] =
+N_char  day_name[8][16] =
     {
-        "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
-    };
-
-N_char  day_name[7][16] =
-    {
-        "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday",
-        "Sunday"
-    };
-
-N_char  month_short[13][4] =    /* abbreviations must be unique! */
-    {
-        "Err",
-        "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+        "Error", "Monday", "Tuesday", "Wednesday",
+        "Thursday", "Friday", "Saturday", "Sunday"
     };
 
 N_char  month_name[13][16] =
     {
-        "Error",
-        "January", "February", "March", "April", "May", "June",
+        "Error", "January", "February", "March", "April", "May", "June",
         "July", "August", "September", "October", "November", "December"
     };
 
-blockdef(rsrc_date_001,16) = "<no date>";   /* short form, exactly 9 chars long */
-blockdef(rsrc_date_002,32) = "<no date>";   /* short form */
-blockdef(rsrc_date_003,64) = "<no date>";   /* verbose form */
+blockdef(rsrc_date_001,16) = "<no date>"; /* exactly 9 chars long */
+blockdef(rsrc_date_002,32) = "<no date>"; /* short form */
+blockdef(rsrc_date_003,64) = "<no date>"; /* verbose form */
 
 /**************************************/
 /* IMPLEMENTATION                     */
@@ -98,13 +99,13 @@ boolean leap(N_int year)
     return((((year % 4) == 0) and ((year % 100) != 0)) or ((year % 400) == 0));
 }
 
-static long year_to_days(N_int year)
+static Z_long year_to_days(N_int year)
 {
     return( year * 365L + (year / 4) - (year / 100) + (year / 400) );
 }
 /****************************************************************************/
 
-N_int encode(N_int yy, N_int mm, N_int dd)
+N_int compress(N_int yy, N_int mm, N_int dd)
 {
     N_int year;
 
@@ -132,7 +133,7 @@ N_int encode(N_int yy, N_int mm, N_int dd)
     return( (yy SHL 9) OR (mm SHL 5) OR dd );
 }
 
-void decode(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd)
+void uncompress(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd)
 {
     if (date > 0)
     {
@@ -157,7 +158,7 @@ void decode(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd)
     }
 }
 
-boolean valid_date(N_int date)
+boolean check_compressed(N_int date)
 {
     N_int cc;
     N_int yy;
@@ -166,13 +167,13 @@ boolean valid_date(N_int date)
 
     if (date > 0)
     {
-        decode(date, &cc, &yy, &mm, &dd);
-        return(date == encode(cc+yy, mm, dd));
+        uncompress(date, &cc, &yy, &mm, &dd);
+        return(date == compress(cc+yy, mm, dd));
     }
     else return(false);
 }
 
-byteptr date_string(N_int date)
+byteptr compressed_to_short(N_int date)
 {
     N_int cc;
     N_int yy;
@@ -184,10 +185,10 @@ byteptr date_string(N_int date)
     if (datestr == NULL) return(NULL);
     if (date > 0)
     {
-        decode(date, &cc, &yy, &mm, &dd);
-        if (date == encode(cc+yy, mm, dd))
+        uncompress(date, &cc, &yy, &mm, &dd);
+        if (date == compress(cc+yy, mm, dd))
         {
-            sprintf((char *)datestr,"%02d-%s-%02d",dd,month_short[mm],yy);
+            sprintf((char *)datestr,"%02d-%.3s-%02d",dd,month_name[mm],yy);
             return(datestr);
         }
     }
@@ -204,7 +205,7 @@ boolean check_date(N_int year, N_int mm, N_int dd)
     return(true);
 }
 
-long calc_days(N_int year, N_int mm, N_int dd)
+Z_long calc_days(N_int year, N_int mm, N_int dd)
 {
     boolean lp;
 
@@ -214,20 +215,29 @@ long calc_days(N_int year, N_int mm, N_int dd)
     return( year_to_days(--year) + days_in_months[lp][mm] + dd );
 }
 
-long dates_difference(N_int year1, N_int mm1, N_int dd1,
+N_int day_of_week(N_int year, N_int mm, N_int dd)
+{
+    Z_long  days;
+
+    days = calc_days(year, mm, dd);
+    if (days > 0L)
+    {
+        days--;
+        days %= 7L;
+        days++;
+    }
+    return( (N_int) days );
+}
+
+Z_long dates_difference(N_int year1, N_int mm1, N_int dd1,
                       N_int year2, N_int mm2, N_int dd2)
 {
     return( calc_days(year2, mm2, dd2) - calc_days(year1, mm1, dd1) );
 }
 
-N_int day_of_week(N_int year, N_int mm, N_int dd)
+void calc_new_date(N_int *year, N_int *mm, N_int *dd, Z_long offset)
 {
-    return( (N_int) ( (calc_days(year, mm, dd) - 1L) % 7L ) );
-}
-
-void calc_new_date(N_int *year, N_int *mm, N_int *dd, long offset)
-{
-    long    days;
+    Z_long  days;
     boolean lp;
 
     if (((days = calc_days(*year, *mm, *dd)) > 0L) and ((days += offset) > 0L))
@@ -261,6 +271,127 @@ void calc_new_date(N_int *year, N_int *mm, N_int *dd, long offset)
 }
 /****************************************************************************/
 
+void date_time_difference
+(
+    Z_long *days, Z_int *hh, Z_int *mm, Z_int *ss,
+    N_int year1, N_int month1, N_int day1, N_int hh1, N_int mm1, N_int ss1,
+    N_int year2, N_int month2, N_int day2, N_int hh2, N_int mm2, N_int ss2
+)
+{
+    Z_long  diff;
+    Z_long  quot;
+    boolean sign;
+
+    *hh = *mm = *ss = 0;
+    if ((ss1 > 59) or (mm1 > 59) or (hh1 > 23) or
+        !check_date(year1,month1,day1))
+    {
+        year1 = month1 = day1 = hh1 = mm1 = ss1 = 0;
+    }
+    if ((ss2 > 59) or (mm2 > 59) or (hh2 > 23) or
+        !check_date(year2,month2,day2))
+    {
+        year2 = month2 = day2 = hh2 = mm2 = ss2 = 0;
+    }
+    diff = ((((hh2 * 60L) + mm2) * 60L) + ss2) -
+           ((((hh1 * 60L) + mm1) * 60L) + ss1);
+    *days = dates_difference(year1, month1, day1, year2, month2, day2);
+    if (*days != 0L)
+    {
+        if (*days > 0L)
+        {
+            if (diff < 0L)
+            {
+                diff += 86400L;
+                (*days)--;
+            }
+        }
+        else
+        {
+            if (diff > 0L)
+            {
+                diff -= 86400L;
+                (*days)++;
+            }
+        }
+    }
+    if (diff != 0L)
+    {
+        sign = false;
+        if (diff < 0L)
+        {
+            sign = true;
+            diff = -diff;
+        }
+        quot = (Z_long) (diff / 60);
+        *ss = diff - quot * 60;
+        diff = quot;
+        quot = (Z_long) (diff / 60);
+        *mm = diff - quot * 60;
+        *hh = quot;
+        if (sign)
+        {
+            *ss = -(*ss);
+            *mm = -(*mm);
+            *hh = -(*hh);
+        }
+    }
+}
+
+void calc_new_date_time
+(
+    N_int *year, N_int *month, N_int *day, N_int *hh, N_int *mm, N_int *ss,
+    Z_long days_offset, Z_long hh_offset, Z_long mm_offset, Z_long ss_offset
+)
+{
+    Z_long  sum;
+    Z_long  quot;
+
+    if ((*ss <= 60) and (*mm <= 60) and (*hh <= 24) and
+        check_date(*year,*month,*day))
+    {
+        sum = ((((*hh * 60L) + *mm) * 60L) + *ss) +
+              ((((hh_offset * 60L) + mm_offset) * 60L) + ss_offset);
+        if (sum < 0L)
+        {
+            quot = (Z_long) (-sum / 86400L);
+            sum += quot * 86400L;
+            days_offset -= quot;
+            if (sum < 0L)
+            {
+                sum += 86400L;
+                days_offset--;
+            }
+        }
+        if (sum > 0L)
+        {
+            quot = (Z_long) (sum / 60);
+            *ss = sum - quot * 60;
+            sum = quot;
+            quot = (Z_long) (sum / 60);
+            *mm = sum - quot * 60;
+            sum = quot;
+            quot = (Z_long) (sum / 24);
+            *hh = sum - quot * 24;
+            days_offset += quot;
+        }
+        else
+        {
+            *hh = *mm = *ss = 0;
+        }
+        calc_new_date(year, month, day, days_offset);
+        if (*year == 0)
+        {
+            *year = *month = *day = *hh = *mm = *ss = 0;
+        }
+    }
+    else
+    {
+        *year = *month = *day = *hh = *mm = *ss = 0;
+    }
+}
+/****************************************************************************/
+
 byteptr date_to_short(N_int year, N_int mm, N_int dd)
 {
     byteptr datestr;
@@ -268,9 +399,9 @@ byteptr date_to_short(N_int year, N_int mm, N_int dd)
     datestr = (byteptr) malloc(32);
     if (datestr == NULL) return(NULL);
     if (check_date(year,mm,dd))
-        sprintf((char *)datestr,"%s %d-%s-%d",
-            day_short[day_of_week(year,mm,dd)],
-            dd,month_short[mm],year);
+        sprintf((char *)datestr,"%.3s %d-%.3s-%d",
+            day_name[day_of_week(year,mm,dd)],
+            dd,month_name[mm],year);
     else
         strcpy((char *)datestr,(char *)rsrc_date_002);
     return(datestr);
@@ -296,7 +427,7 @@ N_int week_number(N_int year, N_int mm, N_int dd)
 {
     N_int first;
 
-    first = day_of_week(year,1,1);
+    first = day_of_week(year,1,1) - 1;
     return( (N_int) ( (dates_difference(year,1,1, year,mm,dd) + first) / 7L ) +
       (first < 4) );
 }
@@ -306,14 +437,14 @@ void first_in_week(N_int week, N_int *year, N_int *mm, N_int *dd)
     N_int first;
 
     *mm = *dd = 1;
-    first = day_of_week(*year,1,1);
+    first = day_of_week(*year,1,1) - 1;
     if (first < 4) week--;
     calc_new_date(year, mm, dd, (week * 7L - first) );
 }
 
 N_int weeks_in_year(N_int year)
 {
-    return(52 + ((day_of_week(year,1,1)==3) or (day_of_week(year,12,31)==3)));
+    return(52 + ((day_of_week(year,1,1)==4) or (day_of_week(year,12,31)==4)));
 }
 /****************************************************************************/
 
@@ -347,29 +478,58 @@ static N_int getval(byteptr src, int len)
     else return(0);
 }
 
-static N_int decode_month(byteptr buf, int len)  /* 0 = unable to decode month */
+N_int decode_day(byteptr buffer, N_int len) /* 0 = unable to decode day */
 {
-    int     i,j;
+    N_int   i,j;
+    N_int   day;
+    boolean same;
+    boolean ok;
+
+    day = 0;
+    ok = true;
+    if (len > 3) len = 3;
+    for ( i=1; ok and (i<8); i++ )
+    {
+        same = true;
+        for ( j=0; same and (j<len); j++ )
+        {
+            same = ( toupper(buffer[j]) == toupper(day_name[i][j]) );
+        }
+        if (same)
+        {
+            if (day)
+                ok = false;
+            else
+                day = i;
+        }
+    }
+    if (ok) return(day);
+    else return(0);
+}
+
+N_int decode_month(byteptr buffer, N_int len) /* 0 = unable to decode month */
+{
+    N_int   i,j;
     N_int   month;
     boolean same;
     boolean ok;
 
     month = 0;
-    ok = (len > 0);
+    ok = true;
     if (len > 3) len = 3;
     for ( i=1; ok and (i<13); i++ )
     {
         same = true;
         for ( j=0; same and (j<len); j++ )
         {
-            same = ( toupper(buf[j]) == toupper(month_short[i][j]) );
+            same = ( toupper(buffer[j]) == toupper(month_name[i][j]) );
         }
         if (same)
         {
             if (month)
                 ok = false;
             else
-                month = (N_int) i;
+                month = i;
         }
     }
     if (ok) return(month);
@@ -489,7 +649,7 @@ boolean decode_date(byteptr buffer, N_int *year, N_int *mm, N_int *dd)
 /**************************************/
 /* CREATED      01.11.93              */
 /**************************************/
-/* MODIFIED     07.12.95              */
+/* MODIFIED     25.05.96              */
 /**************************************/
 /* COPYRIGHT    Steffen Beyer         */
 /**************************************/
