@@ -18,17 +18,29 @@
 
 boolean leap(N_int year);
 
-N_int   encode(N_int yy, N_int mm, N_int dd);
-void    decode(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd);
-boolean valid_date(N_int date);
-byteptr date_string(N_int date);    /* exactly 9 chars long */
+N_int   compress(N_int yy, N_int mm, N_int dd);
+void    uncompress(N_int date, N_int *cc, N_int *yy, N_int *mm, N_int *dd);
+boolean check_compressed(N_int date);
+byteptr compressed_to_short(N_int date);
 
 boolean check_date(N_int year, N_int mm, N_int dd);
-long    calc_days(N_int year, N_int mm, N_int dd);
-long    dates_difference(N_int year1, N_int mm1, N_int dd1,
-                         N_int year2, N_int mm2, N_int dd2);
+Z_long  calc_days(N_int year, N_int mm, N_int dd);
 N_int   day_of_week(N_int year, N_int mm, N_int dd);
-void    calc_new_date(N_int *year, N_int *mm, N_int *dd, long offset);
+Z_long  dates_difference(N_int year1, N_int mm1, N_int dd1,
+                         N_int year2, N_int mm2, N_int dd2);
+void    calc_new_date(N_int *year, N_int *mm, N_int *dd, Z_long offset);
+
+void    date_time_difference
+(
+    Z_long *days, Z_int *hh, Z_int *mm, Z_int *ss,
+    N_int year1, N_int month1, N_int day1, N_int hh1, N_int mm1, N_int ss1,
+    N_int year2, N_int month2, N_int day2, N_int hh2, N_int mm2, N_int ss2
+);
+void    calc_new_date_time
+(
+    N_int *year, N_int *month, N_int *day, N_int *hh, N_int *mm, N_int *ss,
+    Z_long days_offset, Z_long hh_offset, Z_long mm_offset, Z_long ss_offset
+);
 
 byteptr date_to_short(N_int year, N_int mm, N_int dd);
 byteptr date_to_string(N_int year, N_int mm, N_int dd);
@@ -37,6 +49,8 @@ N_int   week_number(N_int year, N_int mm, N_int dd);
 void    first_in_week(N_int week, N_int *year, N_int *mm, N_int *dd);
 N_int   weeks_in_year(N_int year);
 
+N_int   decode_day(byteptr buffer, N_int len);
+N_int   decode_month(byteptr buffer, N_int len);
 boolean decode_date(byteptr buffer, N_int *year, N_int *mm, N_int *dd);
 
 /**************************************/
@@ -51,42 +65,25 @@ extern N_int   month_length[2][13];
     };
 */
 
-extern N_char  day_short[7][4];
+extern N_char  day_name[8][16];
 /*
     {
-        "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
-    };
-*/
-
-extern N_char  day_name[7][16];
-/*
-    {
-        "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday",
-        "Sunday"
-    };
-*/
-
-extern N_char  month_short[13][4];     /* abbreviations must be unique! */
-/*
-    {
-        "Err",
-        "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+        "Error", "Monday", "Tuesday", "Wednesday",
+        "Thursday", "Friday", "Saturday", "Sunday"
     };
 */
 
 extern N_char  month_name[13][16];
 /*
     {
-        "Error",
-        "January", "February", "March", "April", "May", "June",
+        "Error", "January", "February", "March", "April", "May", "June",
         "July", "August", "September", "October", "November", "December"
     };
 */
 
-extern blockdef(rsrc_date_001,16); /* "<no date>";   short form, exactly 9 chars long */
-extern blockdef(rsrc_date_002,32); /* "<no date>";   short form */
-extern blockdef(rsrc_date_003,64); /* "<no date>";   verbose form */
+extern blockdef(rsrc_date_001,16); /* = "<no date>"; exactly 9 chars long */
+extern blockdef(rsrc_date_002,32); /* = "<no date>"; short form */
+extern blockdef(rsrc_date_003,64); /* = "<no date>"; verbose form */
 
 /**************************************/
 /* IMPLEMENTATION                     */
@@ -97,7 +94,7 @@ extern blockdef(rsrc_date_003,64); /* "<no date>";   verbose form */
 /**************************************/
 /* CREATED      01.11.93              */
 /**************************************/
-/* MODIFIED     07.12.95              */
+/* MODIFIED     25.05.96              */
 /**************************************/
 /* COPYRIGHT    Steffen Beyer         */
 /**************************************/
